@@ -229,8 +229,7 @@ static int copy_one_inline(int fd, struct btrfs_path *path, u64 pos)
 	fi = btrfs_item_ptr(leaf, path->slots[0],
 			    struct btrfs_file_extent_item);
 	ptr = btrfs_file_extent_inline_start(fi);
-	len = btrfs_file_extent_inline_item_len(leaf,
-					btrfs_item_nr(path->slots[0]));
+	len = btrfs_file_extent_inline_len(leaf, path->slots[0], fi);
 	read_extent_buffer(leaf, buf, ptr, len);
 
 	compress = btrfs_file_extent_compression(leaf, fi);
@@ -297,7 +296,7 @@ static int copy_one_extent(struct btrfs_root *root, int fd,
 	ram_size = btrfs_file_extent_ram_bytes(leaf, fi);
 	offset = btrfs_file_extent_offset(leaf, fi);
 	num_bytes = btrfs_file_extent_num_bytes(leaf, fi);
-	size_left = num_bytes;
+	size_left = disk_size;
 	if (compress == BTRFS_COMPRESS_NONE)
 		bytenr += offset;
 
@@ -376,7 +375,7 @@ again:
 		goto out;
 	}
 
-	ret = decompress(inbuf, outbuf, num_bytes, &ram_size, compress);
+	ret = decompress(inbuf, outbuf, disk_size, &ram_size, compress);
 	if (ret) {
 		num_copies = btrfs_num_copies(&root->fs_info->mapping_tree,
 					      bytenr, length);
